<#ibizinclude>
../../@MACRO/LANG_FUN.ftl
</#ibizinclude>
<template>
  <gantt-elastic :class="ganttClass" :tasks="tasks" :options="getOptions(this.locale)" :dynamic-style="dynamicStyle" @taskList-item-dblclick="taskClick" @task-item-expand="taskItemExpand">
  </gantt-elastic>
</template>
<#assign import_block>
import GanttElastic from "ibiz-gantt-elastic/src/GanttElastic.vue";
</#assign>

<#assign component_block>
GanttElastic,
</#assign>

<#ibizinclude>
../@MACRO/CONTROL/CONTROL_HEADER-BASE.vue.ftl
</#ibizinclude>

    /**
     * 显示处理提示
     *
     * @type {boolean}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop({ default: true }) public showBusyIndicator?: boolean;

    /**
     * 部件行为--load
     *
     * @type {string}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop() public loadAction!: string;

    /**
     * 部件样式名
     *
     * @public
     * @type {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public ganttClass: string = "gantt";

     /**
     * 语言
     *
     * @public
     * @type {string}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */  
    public locale: string = 'zh-CN';

     /**
     * 语言资源
     *
     * @public
     * @type {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */   
    public localeZH: any =  {
        weekdays: ['星期一','星期二','星期三','星期四','星期五','星期六','星期日'],
        months: ['一月', '二月', '三月', '四月', '五月', '六月', '七月', '八月', '九月', '十月', '十一月', '十二月'],
        Now: "现在",
        "X-Scale": "时间颗粒度",
        "Display task list": "显示列表",
        "Before/After": "数据范围"
    }

    /**
     * 配置参数
     *
     * @public
     * @type {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public options: any = {
        isflex: true,
        maxRows: 1000,
        dataType: 'treegrid',
        title: {
            label: ""
        },
        header: {
            yScole: false,
            taskListWidth: false,
        },
        calendar: {
            hour: {
                display: false
            }
        },
        chart: {
            progress: {
                bar: false
            },
            expander: {
                display: true
            },
        },
        taskList: {
            expander: {
                straight: false
            },
            labelField: 'text',
            columns: [
                <#if ctrl.getPSDETreeColumns()??>
                <#list ctrl.getPSDETreeColumns() as column>
                {
                    label: '${column.getCaption()}',
                    value: (task: any) => {
                        <#if column.render??> 
                        ${column.render.code}
                        <#else>
                        return this.getColumnValue(task, '<#if column.getName() == "begin">start<#else>${column.getName()}</#if>')
                        </#if>
                    },
                    <#if column.getName() == "text">
                    expander: true,
                    </#if>
                    width: <#if column.getWidth() gt 0>${column.getWidth()}<#else>100</#if>,
                    <#if column.isHideDefault()>
                    hidden: true,
                    </#if>
                    <#if column.getHeaderPSSysCss()??>
                    headerCls: '${column.getHeaderPSSysCss().getCssName()}',
                    </#if>
                    <#if column.getCellPSSysCss()??>
                    cellCls: '${column.getCellPSSysCss().getCssName()}',
                    </#if>
                },
                </#list>
                </#if>
            ]
        }
    };

    /**
     * 样式
     *
     * @public
     * @type {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public dynamicStyle: any = {};

    /**
     * 日程事件集合
     *
     * @public
     * @type {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public tasks: any[] = [];

    /**
     * 监听语言变化
     *
     * @public
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Watch('$i18n.locale')
    public onLocaleChange(newval: any, val: any) {
        this.locale = newval;
    }

    /**
     * 获取列属性值
     *
     * @public
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public getColumnValue(task: any, field: string) {
        <#if ctrl.getPSDETreeNodes()??>
        <#list ctrl.getPSDETreeNodes() as node>
        if(Object.is(task.id.split(';')[0], '${srfjavastring(node.getNodeType())}')) {
            <#if node.getPSDETreeNodeDataItems()??>
            <#list node.getPSDETreeNodeDataItems() as dataItem>
            <#if dataItem.getDataItemParam0()?? && dataItem.getDataItemParam0().getPSCodeList && dataItem.getDataItemParam0().getPSCodeList()??>
            CODELIST
            <#assign codelist = dataItem.getDataItemParam0().getPSCodeList()/>
            if (Object.is(field, '${dataItem.name}')) {
                let items: any[] = this.$store.getters.getCodeList('${codelist.codeName}');
                if(items) {
                    let item = this.getCodeListItem(items, task[field]);
                    return item.text;
                }
            }
            </#if>
            <#if dataItem.render??>
            if (Object.is(field, '${dataItem.name}')) {
                ${dataItem.render.code}
            }
            </#if>
            </#list>
            </#if>
            return task[field];
        }
        </#list>
        </#if>
    }

    /**
     * 获取代码项
     *
     * @public
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public getCodeListItem(items: any[], val: any) {
        for(let i = 0; i < items.length; i++) {
            if(Object.is(items[i].value, val)) {
                return items[i];
            }
        }
    }

    /**
     * 获取部件参数
     *
     * @returns {any}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public getOptions() {
        if(Object.is(this.locale, 'zh-CN')) {
            return { locale: this.localeZH, ...this.options };
        }
        return this.options;
    }

    /**
     * 搜索获取日程事件
     *
     * @param {*} $event 日期信息
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public load(task: any = {}) {
        const params: any = {
            srfnodeid: task && task.id ? task.id : "#",
            srfnodefilter: ''
        };
        let tempViewParams:any = JSON.parse(JSON.stringify(this.viewparams));
        let curNode:any = {}; 
        Util.deepObjectMerge(curNode, task);
        let tempContext:any = this.computecurNodeContext(curNode);
        if(curNode && curNode.srfparentdename) {
            Object.assign(tempContext,{ srfparentdename: curNode.srfparentdename });
            Object.assign(tempViewParams,{ srfparentdename: curNode.srfparentdename });
        }
        if(curNode && curNode.srfparentkey) {
            Object.assign(tempContext,{ srfparentkey: curNode.srfparentkey });
            Object.assign(tempViewParams,{ srfparentkey: curNode.srfparentkey });
        }
        Object.assign(params,{viewparams:tempViewParams});
        this.service.getNodes(tempContext,params).then((response: any) => {
            if (!response || response.status !== 200) {
                this.$Notice.error({ title: "错误", desc: response.info });
                return;
            }
            this.tasks = [...this.tasks, ...response.data];
            response.data.forEach((item: any) => {
                if(!item.collapsed) {
                    this.load(item);
                }
            })
            this.$emit("load", this.tasks);
        }).catch((response: any) => {
            if (response && response.status === 401) {
                return;
            }
            this.$Notice.error({ title: "错误", desc: response.info });
        });
    }

    /**
     * 计算当前节点的上下文
     *
     * @param {*} curNode 当前节点
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public computecurNodeContext(curNode:any){
        let tempContext:any = {};
        if(curNode && curNode.data && curNode.data.srfappctx){
            tempContext = JSON.parse(JSON.stringify(curNode.data.srfappctx));
        }else{
            tempContext = JSON.parse(JSON.stringify(this.context));
        }
        return tempContext;
    }

    /**
     * 节点展开
     *
     * @param {*} task 当前节点
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public taskItemExpand(task: any) {
        if(!task.collapsed) {
            let index: number = this.tasks.findIndex((item: any) => Object.is(task.id, item.parentId));
            if(index < 0) {
                this.load(task);
            }
        }
    }

    /**
     * 获取编辑视图信息
     *
     * @param {*} $event 事件信息
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public getEditView(deName: string) {
        let view: any = {};
        switch(deName){
<#if ctrl.getPSDETreeNodes()??>
  <#list ctrl.getPSDETreeNodes() as ganttItem>
    <#if ganttItem.getPSAppDataEntity()??>
        <#assign _appde = ganttItem.getPSAppDataEntity() />
            case "${_appde.getCodeName()?lower_case}":
                <#if ganttItem.getNavPSAppView()??>
                <#assign editview = ganttItem.getNavPSAppView()/>
                view = {
                    viewname: '${srffilepath2(editview.getCodeName())}', 
                    height: ${editview.getHeight()?c}, 
                    width: ${editview.getWidth()?c},
                    title: this.<@getViewLanguageTitle editview />,
                    placement: '${editview.getOpenMode()}',
                    deResParameters: <#rt>
    <#if editview.isPSDEView()>
      [<#t>
      <#if editview.getPSAppDERSPathCount() gt 0>
        <#list editview.getPSAppDERSPath(editview.getPSAppDERSPathCount() - 1) as deRSPath>
        <#assign majorPSAppDataEntity = deRSPath.getMajorPSAppDataEntity()/><#t>
          { pathName: '${srfpluralize(majorPSAppDataEntity.codeName)?lower_case}', parameterName: '${majorPSAppDataEntity.getCodeName()?lower_case}' }, <#t>
        </#list>
      </#if>
      ],<#lt>
    <#else>
      [],<#lt>
    </#if>
                    parameters: <#rt>
    [<#t>
    <#if editview.isPSDEView()>
      <#assign appDataEntity = editview.getPSAppDataEntity()/>
      { pathName: '${srfpluralize(appDataEntity.codeName)?lower_case}', parameterName: '${appDataEntity.getCodeName()?lower_case}' }, <#t>
      { pathName: '${editview.getPSDEViewCodeName()?lower_case}', parameterName: '${editview.getPSDEViewCodeName()?lower_case}' } <#t>
    <#else>
      { pathName: '${editview.getCodeName()?lower_case}', parameterName: '${editview.getCodeName()?lower_case}' } <#t>
    </#if>
    ],<#lt>
                };
                </#if>
                break;
    </#if>
  </#list>
</#if>
        }
        return view;
    }

    /**
     * 刷新
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public refresh() {
        this.load();
    }



    /**
     * 选中的数据
     *
     * @returns {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public selections: any[] = [];

    /**
     * 获取多项数据
     *
     * @returns {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public getDatas(): any[] {
        return this.selections;
    }

    /**
     * 获取单项树
     *
     * @returns {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public getData(): any {
        return null;
    }
             
    /**
     * vue 生命周期
     *
     * @returns
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public created() {
        this.locale = this.$i18n.locale;
        this.load();
    }

    /**
     * 点击事件
     *
     * @returns
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public taskClick({event, data}: {event: any, data: any}) {
        let view: any = {};
        let _context: any = Object.assign({},this.context);
        let key: string = data.id.split(';')[0];
        switch(key) {
<#if ctrl.getPSDETreeNodes()??>
  <#list ctrl.getPSDETreeNodes() as ganttItem>
    <#if ganttItem.getPSAppDataEntity()??>
      <#assign _appde = ganttItem.getPSAppDataEntity() />
            case "${ganttItem.getNodeType()}":
                _context.${_appde.getCodeName()?lower_case} = data.${_appde.getCodeName()?lower_case};
                view = this.getEditView("${_appde.getCodeName()?lower_case}");
                break;
    </#if>
  </#list>
</#if>
        }
        if (!view.viewname) {
            return;
        }
        // 根据打开模式打开视图
        if (Object.is(view.placement, 'INDEXVIEWTAB') || Object.is(view.placement, '')) {
            const routePath = this.$viewTool.buildUpRoutePath(this.$route, this.context, view.deResParameters, view.parameters, [JSON.parse(JSON.stringify(_context))] , JSON.parse(JSON.stringify(this.viewparams)));
            this.$router.push(routePath);
        } else {
            let container: Subject<any> = new Subject();
            if (Object.is(view.placement, 'POPOVER')) {
                container = this.$apppopover.openPop(data, view,JSON.parse(JSON.stringify(_context)),  JSON.parse(JSON.stringify(this.viewparams)));
            } else if (Object.is(view.placement, 'POPUPMODAL')) {
                container = this.$appmodal.openModal(view,  JSON.parse(JSON.stringify(_context)),  JSON.parse(JSON.stringify(this.viewparams)));
            } else if (view.placement.startsWith('DRAWER')) {
                container = this.$appdrawer.openDrawer(view,  JSON.parse(JSON.stringify(_context)),  JSON.parse(JSON.stringify(this.viewparams)));
            }
            container.subscribe((result: any) => {
                if (!result || !Object.is(result.ret, 'OK')) {
                    return;
                }
                this.refresh();
            });
        }
    }

<#ibizinclude>
../@MACRO/CONTROL/CONTROL_BOTTOM-BASE.vue.ftl
</#ibizinclude>

<#ibizinclude>
../@MACRO/CONTROL/CONTROL-BASE.style.ftl
</#ibizinclude>